# This file has been autogenerated by the pywayland scanner

# Copyright 2014 © Stephen "Lyude" Chandler Paul
# Copyright 2015-2016 © Red Hat, Inc.
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice (including the
# next paragraph) shall be included in all copies or substantial
# portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
# BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
# ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSeat
from .wayland import WlSeatProxy
from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class ZwpTabletManagerV2(Interface):
    """Controller object for graphic tablet devices

    An object that provides access to the graphics tablets available on this
    system. All tablets are associated with a seat, to get access to the actual
    tablets, use wp_tablet_manager.get_tablet_seat.
    """

    name = "zwp_tablet_manager_v2"
    version = 1


class ZwpTabletPadGroupV2(Interface):
    """A set of buttons, rings and strips

    A pad group describes a distinct (sub)set of buttons, rings and strips
    present in the tablet. The criteria of this grouping is usually positional,
    eg. if a tablet has buttons on the left and right side, 2 groups will be
    presented. The physical arrangement of groups is undisclosed and may change
    on the fly.

    Pad groups will announce their features during pad initialization. Between
    the corresponding wp_tablet_pad.group event and wp_tablet_pad_group.done,
    the pad group will announce the buttons, rings and strips contained in it,
    plus the number of supported modes.

    Modes are a mechanism to allow multiple groups of actions for every element
    in the pad group. The number of groups and available modes in each is
    persistent across device plugs. The current mode is user-switchable, it
    will be announced through the wp_tablet_pad_group.mode_switch event both
    whenever it is switched, and after wp_tablet_pad.enter.

    The current mode logically applies to all elements in the pad group,
    although it is at clients' discretion whether to actually perform different
    actions, and/or issue the respective .set_feedback requests to notify the
    compositor. See the wp_tablet_pad_group.mode_switch event for more details.
    """

    name = "zwp_tablet_pad_group_v2"
    version = 1


class ZwpTabletPadRingV2(Interface):
    """Pad ring

    A circular interaction area, such as the touch ring on the Wacom Intuos Pro
    series tablets.

    Events on a ring are logically grouped by the wl_tablet_pad_ring.frame
    event.
    """

    name = "zwp_tablet_pad_ring_v2"
    version = 1

    class source(enum.IntEnum):
        finger = 1


class ZwpTabletPadStripV2(Interface):
    """Pad strip

    A linear interaction area, such as the strips found in Wacom Cintiq models.

    Events on a strip are logically grouped by the wl_tablet_pad_strip.frame
    event.
    """

    name = "zwp_tablet_pad_strip_v2"
    version = 1

    class source(enum.IntEnum):
        finger = 1


class ZwpTabletPadV2(Interface):
    """A set of buttons, rings and strips

    A pad device is a set of buttons, rings and strips usually physically
    present on the tablet device itself. Some exceptions exist where the pad
    device is physically detached, e.g. the Wacom ExpressKey Remote.

    Pad devices have no axes that control the cursor and are generally
    auxiliary devices to the tool devices used on the tablet surface.

    A pad device has a number of static characteristics, e.g. the number of
    rings. These capabilities are sent in an event sequence after the
    wp_tablet_seat.pad_added event before any actual events from this pad. This
    initial event sequence is terminated by a wp_tablet_pad.done event.

    All pad features (buttons, rings and strips) are logically divided into
    groups and all pads have at least one group. The available groups are
    notified through the wp_tablet_pad.group event; the compositor will emit
    one event per group before emitting wp_tablet_pad.done.

    Groups may have multiple modes. Modes allow clients to map multiple actions
    to a single pad feature. Only one mode can be active per group, although
    different groups may have different active modes.
    """

    name = "zwp_tablet_pad_v2"
    version = 1

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1


class ZwpTabletSeatV2(Interface):
    """Controller object for graphic tablet devices of a seat

    An object that provides access to the graphics tablets available on this
    seat. After binding to this interface, the compositor sends a set of
    wp_tablet_seat.tablet_added and wp_tablet_seat.tool_added events.
    """

    name = "zwp_tablet_seat_v2"
    version = 1


class ZwpTabletToolV2(Interface):
    """A physical tablet tool

    An object that represents a physical tool that has been, or is currently in
    use with a tablet in this seat. Each wp_tablet_tool object stays valid
    until the client destroys it; the compositor reuses the wp_tablet_tool
    object to indicate that the object's respective physical tool has come into
    proximity of a tablet again.

    A wp_tablet_tool object's relation to a physical tool depends on the
    tablet's ability to report serial numbers. If the tablet supports this
    capability, then the object represents a specific physical tool and can be
    identified even when used on multiple tablets.

    A tablet tool has a number of static characteristics, e.g. tool type,
    hardware_serial and capabilities. These capabilities are sent in an event
    sequence after the wp_tablet_seat.tool_added event before any actual events
    from this tool. This initial event sequence is terminated by a
    wp_tablet_tool.done event.

    Tablet tool events are grouped by wp_tablet_tool.frame events. Any events
    received before a wp_tablet_tool.frame event should be considered part of
    the same hardware state change.
    """

    name = "zwp_tablet_tool_v2"
    version = 1

    class type(enum.IntEnum):
        pen = 0x140
        eraser = 0x141
        brush = 0x142
        pencil = 0x143
        airbrush = 0x144
        finger = 0x145
        mouse = 0x146
        lens = 0x147

    class capability(enum.IntEnum):
        tilt = 1
        pressure = 2
        distance = 3
        rotation = 4
        slider = 5
        wheel = 6

    class button_state(enum.IntEnum):
        released = 0
        pressed = 1

    class error(enum.IntEnum):
        role = 0


class ZwpTabletV2(Interface):
    """Graphics tablet device

    The wp_tablet interface represents one graphics tablet device. The tablet
    interface itself does not generate events; all events are generated by
    wp_tablet_tool objects when in proximity above a tablet.

    A tablet has a number of static characteristics, e.g. device name and
    pid/vid. These capabilities are sent in an event sequence after the
    wp_tablet_seat.tablet_added event. This initial event sequence is
    terminated by a wp_tablet.done event.
    """

    name = "zwp_tablet_v2"
    version = 1


class ZwpTabletManagerV2Resource(Resource[ZwpTabletManagerV2]):
    interface = ZwpTabletManagerV2


class ZwpTabletPadGroupV2Resource(Resource[ZwpTabletPadGroupV2]):
    interface = ZwpTabletPadGroupV2

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Array),
    )
    def buttons(self, buttons: list) -> None:
        """Buttons announced

        Sent on wp_tablet_pad_group initialization to announce the available
        buttons in the group. Button indices start at 0, a button may only be
        in one group at a time.

        This event is first sent in the initial burst of events before the
        wp_tablet_pad_group.done event.

        Some buttons are reserved by the compositor. These buttons may not be
        assigned to any wp_tablet_pad_group. Compositors may broadcast this
        event in the case of changes to the mapping of these reserved buttons.
        If the compositor happens to reserve all buttons in a group, this event
        will be sent with an empty array.

        :param buttons:
            buttons in this group
        :type buttons:
            `ArgumentType.Array`
        """
        self._post_event(0, buttons)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadRingV2),
    )
    def ring(self, ring: ZwpTabletPadRingV2) -> None:
        """Ring announced

        Sent on wp_tablet_pad_group initialization to announce available rings.
        One event is sent for each ring available on this pad group.

        This event is sent in the initial burst of events before the
        wp_tablet_pad_group.done event.

        :param ring:
        :type ring:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadRingV2`
        """
        self._post_event(1, ring)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadStripV2),
    )
    def strip(self, strip: ZwpTabletPadStripV2) -> None:
        """Strip announced

        Sent on wp_tablet_pad initialization to announce available strips. One
        event is sent for each strip available on this pad group.

        This event is sent in the initial burst of events before the
        wp_tablet_pad_group.done event.

        :param strip:
        :type strip:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadStripV2`
        """
        self._post_event(2, strip)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Uint),
    )
    def modes(self, modes: int) -> None:
        """Mode-switch ability announced

        Sent on wp_tablet_pad_group initialization to announce that the pad
        group may switch between modes. A client may use a mode to store a
        specific configuration for buttons, rings and strips and use the
        wl_tablet_pad_group.mode_switch event to toggle between these
        configurations. Mode indices start at 0.

        Switching modes is compositor-dependent. See the
        wp_tablet_pad_group.mode_switch event for more details.

        This event is sent in the initial burst of events before the
        wp_tablet_pad_group.done event. This event is only sent when more than
        more than one mode is available.

        :param modes:
            the number of modes
        :type modes:
            `ArgumentType.Uint`
        """
        self._post_event(3, modes)

    @ZwpTabletPadGroupV2.event()
    def done(self) -> None:
        """Tablet group description events sequence complete

        This event is sent immediately to signal the end of the initial burst
        of descriptive events. A client may consider the static description of
        the tablet to be complete and finalize initialization of the tablet
        group.
        """
        self._post_event(4)

    @ZwpTabletPadGroupV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def mode_switch(self, time: int, serial: int, mode: int) -> None:
        """Mode switch event

        Notification that the mode was switched.

        A mode applies to all buttons, rings and strips in a group
        simultaneously, but a client is not required to assign different
        actions for each mode. For example, a client may have mode-specific
        button mappings but map the ring to vertical scrolling in all modes.
        Mode indices start at 0.

        Switching modes is compositor-dependent. The compositor may provide
        visual cues to the client about the mode, e.g. by toggling LEDs on the
        tablet device. Mode-switching may be software-controlled or controlled
        by one or more physical buttons. For example, on a Wacom Intuos Pro,
        the button inside the ring may be assigned to switch between modes.

        The compositor will also send this event after wp_tablet_pad.enter on
        each group in order to notify of the current mode. Groups that only
        feature one mode will use mode=0 when emitting this event.

        If a button action in the new mode differs from the action in the
        previous mode, the client should immediately issue a
        wp_tablet_pad.set_feedback request for each changed button.

        If a ring or strip action in the new mode differs from the action in
        the previous mode, the client should immediately issue a
        wp_tablet_ring.set_feedback or wp_tablet_strip.set_feedback request for
        each changed ring or strip.

        :param time:
            the time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param mode:
            the new mode of the pad
        :type mode:
            `ArgumentType.Uint`
        """
        self._post_event(5, time, serial, mode)


class ZwpTabletPadRingV2Resource(Resource[ZwpTabletPadRingV2]):
    interface = ZwpTabletPadRingV2

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Uint),
    )
    def source(self, source: int) -> None:
        """Ring event source

        Source information for ring events.

        This event does not occur on its own. It is sent before a
        wp_tablet_pad_ring.frame event and carries the source information for
        all events within that frame.

        The source specifies how this event was generated. If the source is
        wp_tablet_pad_ring.source.finger, a wp_tablet_pad_ring.stop event will
        be sent when the user lifts the finger off the device.

        This event is optional. If the source is unknown for an interaction, no
        event is sent.

        :param source:
            the event source
        :type source:
            `ArgumentType.Uint`
        """
        self._post_event(0, source)

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Fixed),
    )
    def angle(self, degrees: float) -> None:
        """Angle changed

        Sent whenever the angle on a ring changes.

        The angle is provided in degrees clockwise from the logical north of
        the ring in the pad's current rotation.

        :param degrees:
            the current angle in degrees
        :type degrees:
            `ArgumentType.Fixed`
        """
        self._post_event(1, degrees)

    @ZwpTabletPadRingV2.event()
    def stop(self) -> None:
        """Interaction stopped

        Stop notification for ring events.

        For some wp_tablet_pad_ring.source types, a wp_tablet_pad_ring.stop
        event is sent to notify a client that the interaction with the ring has
        terminated. This enables the client to implement kinetic scrolling. See
        the wp_tablet_pad_ring.source documentation for information on when
        this event may be generated.

        Any wp_tablet_pad_ring.angle events with the same source after this
        event should be considered as the start of a new interaction.
        """
        self._post_event(2)

    @ZwpTabletPadRingV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """End of a ring event sequence

        Indicates the end of a set of ring events that logically belong
        together. A client is expected to accumulate the data in all events
        within the frame before proceeding.

        All wp_tablet_pad_ring events before a wp_tablet_pad_ring.frame event
        belong logically together. For example, on termination of a finger
        interaction on a ring the compositor will send a
        wp_tablet_pad_ring.source event, a wp_tablet_pad_ring.stop event and a
        wp_tablet_pad_ring.frame event.

        A wp_tablet_pad_ring.frame event is sent for every logical event group,
        even if the group only contains a single wp_tablet_pad_ring event.
        Specifically, a client may get a sequence: angle, frame, angle, frame,
        etc.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(3, time)


class ZwpTabletPadStripV2Resource(Resource[ZwpTabletPadStripV2]):
    interface = ZwpTabletPadStripV2

    @ZwpTabletPadStripV2.event(
        Argument(ArgumentType.Uint),
    )
    def source(self, source: int) -> None:
        """Strip event source

        Source information for strip events.

        This event does not occur on its own. It is sent before a
        wp_tablet_pad_strip.frame event and carries the source information for
        all events within that frame.

        The source specifies how this event was generated. If the source is
        wp_tablet_pad_strip.source.finger, a wp_tablet_pad_strip.stop event
        will be sent when the user lifts their finger off the device.

        This event is optional. If the source is unknown for an interaction, no
        event is sent.

        :param source:
            the event source
        :type source:
            `ArgumentType.Uint`
        """
        self._post_event(0, source)

    @ZwpTabletPadStripV2.event(
        Argument(ArgumentType.Uint),
    )
    def position(self, position: int) -> None:
        """Position changed

        Sent whenever the position on a strip changes.

        The position is normalized to a range of [0, 65535], the 0-value
        represents the top-most and/or left-most position of the strip in the
        pad's current rotation.

        :param position:
            the current position
        :type position:
            `ArgumentType.Uint`
        """
        self._post_event(1, position)

    @ZwpTabletPadStripV2.event()
    def stop(self) -> None:
        """Interaction stopped

        Stop notification for strip events.

        For some wp_tablet_pad_strip.source types, a wp_tablet_pad_strip.stop
        event is sent to notify a client that the interaction with the strip
        has terminated. This enables the client to implement kinetic scrolling.
        See the wp_tablet_pad_strip.source documentation for information on
        when this event may be generated.

        Any wp_tablet_pad_strip.position events with the same source after this
        event should be considered as the start of a new interaction.
        """
        self._post_event(2)

    @ZwpTabletPadStripV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """End of a strip event sequence

        Indicates the end of a set of events that represent one logical
        hardware strip event. A client is expected to accumulate the data in
        all events within the frame before proceeding.

        All wp_tablet_pad_strip events before a wp_tablet_pad_strip.frame event
        belong logically together. For example, on termination of a finger
        interaction on a strip the compositor will send a
        wp_tablet_pad_strip.source event, a wp_tablet_pad_strip.stop event and
        a wp_tablet_pad_strip.frame event.

        A wp_tablet_pad_strip.frame event is sent for every logical event
        group, even if the group only contains a single wp_tablet_pad_strip
        event. Specifically, a client may get a sequence: position, frame,
        position, frame, etc.

        :param time:
            timestamp with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(3, time)


class ZwpTabletPadV2Resource(Resource[ZwpTabletPadV2]):
    interface = ZwpTabletPadV2

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadGroupV2),
    )
    def group(self, pad_group: ZwpTabletPadGroupV2) -> None:
        """Group announced

        Sent on wp_tablet_pad initialization to announce available groups. One
        event is sent for each pad group available.

        This event is sent in the initial burst of events before the
        wp_tablet_pad.done event. At least one group will be announced.

        :param pad_group:
        :type pad_group:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadGroupV2`
        """
        self._post_event(0, pad_group)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.String),
    )
    def path(self, path: str) -> None:
        """Path to the device

        A system-specific device path that indicates which device is behind
        this wp_tablet_pad. This information may be used to gather additional
        information about the device, e.g. through libwacom.

        The format of the path is unspecified, it may be a device node, a sysfs
        path, or some other identifier. It is up to the client to identify the
        string provided.

        This event is sent in the initial burst of events before the
        wp_tablet_pad.done event.

        :param path:
            path to local device
        :type path:
            `ArgumentType.String`
        """
        self._post_event(1, path)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
    )
    def buttons(self, buttons: int) -> None:
        """Buttons announced

        Sent on wp_tablet_pad initialization to announce the available buttons.

        This event is sent in the initial burst of events before the
        wp_tablet_pad.done event. This event is only sent when at least one
        button is available.

        :param buttons:
            the number of buttons
        :type buttons:
            `ArgumentType.Uint`
        """
        self._post_event(2, buttons)

    @ZwpTabletPadV2.event()
    def done(self) -> None:
        """Pad description event sequence complete

        This event signals the end of the initial burst of descriptive events.
        A client may consider the static description of the pad to be complete
        and finalize initialization of the pad.
        """
        self._post_event(3)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, time: int, button: int, state: int) -> None:
        """Physical button state

        Sent whenever the physical state of a button changes.

        :param time:
            the time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        :param button:
            the index of the button that changed state
        :type button:
            `ArgumentType.Uint`
        :param state:
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(4, time, button, state)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=ZwpTabletV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def enter(self, serial: int, tablet: ZwpTabletV2, surface: WlSurface) -> None:
        """Enter event

        Notification that this pad is focused on the specified surface.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param tablet:
            the tablet the pad is attached to
        :type tablet:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletV2`
        :param surface:
            surface the pad is focused on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(5, serial, tablet, surface)

    @ZwpTabletPadV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def leave(self, serial: int, surface: WlSurface) -> None:
        """Leave event

        Notification that this pad is no longer focused on the specified
        surface.

        :param serial:
            serial number of the leave event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
            surface the pad is no longer focused on
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(6, serial, surface)

    @ZwpTabletPadV2.event()
    def removed(self) -> None:
        """Pad removed event

        Sent when the pad has been removed from the system. When a tablet is
        removed its pad(s) will be removed too.

        When this event is received, the client must destroy all rings, strips
        and groups that were offered by this pad, and issue
        wp_tablet_pad.destroy the pad itself.
        """
        self._post_event(7)


class ZwpTabletSeatV2Resource(Resource[ZwpTabletSeatV2]):
    interface = ZwpTabletSeatV2

    @ZwpTabletSeatV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletV2),
    )
    def tablet_added(self, id: ZwpTabletV2) -> None:
        """New device notification

        This event is sent whenever a new tablet becomes available on this
        seat. This event only provides the object id of the tablet, any static
        information about the tablet (device name, vid/pid, etc.) is sent
        through the wp_tablet interface.

        :param id:
            the newly added graphics tablet
        :type id:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletV2`
        """
        self._post_event(0, id)

    @ZwpTabletSeatV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletToolV2),
    )
    def tool_added(self, id: ZwpTabletToolV2) -> None:
        """A new tool has been used with a tablet

        This event is sent whenever a tool that has not previously been used
        with a tablet comes into use. This event only provides the object id of
        the tool; any static information about the tool (capabilities, type,
        etc.) is sent through the wp_tablet_tool interface.

        :param id:
            the newly added tablet tool
        :type id:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletToolV2`
        """
        self._post_event(1, id)

    @ZwpTabletSeatV2.event(
        Argument(ArgumentType.NewId, interface=ZwpTabletPadV2),
    )
    def pad_added(self, id: ZwpTabletPadV2) -> None:
        """New pad notification

        This event is sent whenever a new pad is known to the system.
        Typically, pads are physically attached to tablets and a pad_added
        event is sent immediately after the wp_tablet_seat.tablet_added.
        However, some standalone pad devices logically attach to tablets at
        runtime, and the client must wait for wp_tablet_pad.enter to know the
        tablet a pad is attached to.

        This event only provides the object id of the pad. All further features
        (buttons, strips, rings) are sent through the wp_tablet_pad interface.

        :param id:
            the newly added pad
        :type id:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletPadV2`
        """
        self._post_event(2, id)


class ZwpTabletToolV2Resource(Resource[ZwpTabletToolV2]):
    interface = ZwpTabletToolV2

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def type(self, tool_type: int) -> None:
        """Tool type

        The tool type is the high-level type of the tool and usually decides
        the interaction expected from this tool.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param tool_type:
            the physical tool type
        :type tool_type:
            `ArgumentType.Uint`
        """
        self._post_event(0, tool_type)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def hardware_serial(self, hardware_serial_hi: int, hardware_serial_lo: int) -> None:
        """Unique hardware serial number of the tool

        If the physical tool can be identified by a unique 64-bit serial
        number, this event notifies the client of this serial number.

        If multiple tablets are available in the same seat and the tool is
        uniquely identifiable by the serial number, that tool may move between
        tablets.

        Otherwise, if the tool has no serial number and this event is missing,
        the tool is tied to the tablet it first comes into proximity with. Even
        if the physical tool is used on multiple tablets, separate
        wp_tablet_tool objects will be created, one per tablet.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param hardware_serial_hi:
            the unique serial number of the tool, most significant bits
        :type hardware_serial_hi:
            `ArgumentType.Uint`
        :param hardware_serial_lo:
            the unique serial number of the tool, least significant bits
        :type hardware_serial_lo:
            `ArgumentType.Uint`
        """
        self._post_event(1, hardware_serial_hi, hardware_serial_lo)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def hardware_id_wacom(self, hardware_id_hi: int, hardware_id_lo: int) -> None:
        """Hardware id notification in wacom's format

        This event notifies the client of a hardware id available on this tool.

        The hardware id is a device-specific 64-bit id that provides extra
        information about the tool in use, beyond the wl_tool.type enumeration.
        The format of the id is specific to tablets made by Wacom Inc. For
        example, the hardware id of a Wacom Grip Pen (a stylus) is 0x802.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param hardware_id_hi:
            the hardware id, most significant bits
        :type hardware_id_hi:
            `ArgumentType.Uint`
        :param hardware_id_lo:
            the hardware id, least significant bits
        :type hardware_id_lo:
            `ArgumentType.Uint`
        """
        self._post_event(2, hardware_id_hi, hardware_id_lo)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def capability(self, capability: int) -> None:
        """Tool capability notification

        This event notifies the client of any capabilities of this tool, beyond
        the main set of x/y axes and tip up/down detection.

        One event is sent for each extra capability available on this tool.

        This event is sent in the initial burst of events before the
        wp_tablet_tool.done event.

        :param capability:
            the capability
        :type capability:
            `ArgumentType.Uint`
        """
        self._post_event(3, capability)

    @ZwpTabletToolV2.event()
    def done(self) -> None:
        """Tool description events sequence complete

        This event signals the end of the initial burst of descriptive events.
        A client may consider the static description of the tool to be complete
        and finalize initialization of the tool.
        """
        self._post_event(4)

    @ZwpTabletToolV2.event()
    def removed(self) -> None:
        """Tool removed

        This event is sent when the tool is removed from the system and will
        send no further events. Should the physical tool come back into
        proximity later, a new wp_tablet_tool object will be created.

        It is compositor-dependent when a tool is removed. A compositor may
        remove a tool on proximity out, tablet removal or any other reason. A
        compositor may also keep a tool alive until shutdown.

        If the tool is currently in proximity, a proximity_out event will be
        sent before the removed event. See wp_tablet_tool.proximity_out for the
        handling of any buttons logically down.

        When this event is received, the client must wp_tablet_tool.destroy the
        object.
        """
        self._post_event(5)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=ZwpTabletV2),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def proximity_in(self, serial: int, tablet: ZwpTabletV2, surface: WlSurface) -> None:
        """Proximity in event

        Notification that this tool is focused on a certain surface.

        This event can be received when the tool has moved from one surface to
        another, or when the tool has come back into proximity above the
        surface.

        If any button is logically down when the tool comes into proximity, the
        respective button event is sent after the proximity_in event but within
        the same frame as the proximity_in event.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param tablet:
            The tablet the tool is in proximity of
        :type tablet:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletV2`
        :param surface:
            The current surface the tablet tool is over
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        """
        self._post_event(6, serial, tablet, surface)

    @ZwpTabletToolV2.event()
    def proximity_out(self) -> None:
        """Proximity out event

        Notification that this tool has either left proximity, or is no longer
        focused on a certain surface.

        When the tablet tool leaves proximity of the tablet, button release
        events are sent for each button that was held down at the time of
        leaving proximity. These events are sent before the proximity_out event
        but within the same wp_tablet.frame.

        If the tool stays within proximity of the tablet, but the focus changes
        from one surface to another, a button release event may not be sent
        until the button is actually released or the tool leaves the proximity
        of the tablet.
        """
        self._post_event(7)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def down(self, serial: int) -> None:
        """Tablet tool is making contact

        Sent whenever the tablet tool comes in contact with the surface of the
        tablet.

        If the tool is already in contact with the tablet when entering the
        input region, the client owning said region will receive a
        wp_tablet.proximity_in event, followed by a wp_tablet.down event and a
        wp_tablet.frame event.

        Note that this event describes logical contact, not physical contact.
        On some devices, a compositor may not consider a tool in logical
        contact until a minimum physical pressure threshold is exceeded.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        """
        self._post_event(8, serial)

    @ZwpTabletToolV2.event()
    def up(self) -> None:
        """Tablet tool is no longer making contact

        Sent whenever the tablet tool stops making contact with the surface of
        the tablet, or when the tablet tool moves out of the input region and
        the compositor grab (if any) is dismissed.

        If the tablet tool moves out of the input region while in contact with
        the surface of the tablet and the compositor does not have an ongoing
        grab on the surface, the client owning said region will receive a
        wp_tablet.up event, followed by a wp_tablet.proximity_out event and a
        wp_tablet.frame event. If the compositor has an ongoing grab on this
        device, this event sequence is sent whenever the grab is dismissed in
        the future.

        Note that this event describes logical contact, not physical contact.
        On some devices, a compositor may not consider a tool out of logical
        contact until physical pressure falls below a specific threshold.
        """
        self._post_event(9)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def motion(self, x: float, y: float) -> None:
        """Motion event

        Sent whenever a tablet tool moves.

        :param x:
            surface-local x coordinate
        :type x:
            `ArgumentType.Fixed`
        :param y:
            surface-local y coordinate
        :type y:
            `ArgumentType.Fixed`
        """
        self._post_event(10, x, y)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def pressure(self, pressure: int) -> None:
        """Pressure change event

        Sent whenever the pressure axis on a tool changes. The value of this
        event is normalized to a value between 0 and 65535.

        Note that pressure may be nonzero even when a tool is not in logical
        contact. See the down and up events for more details.

        :param pressure:
            The current pressure value
        :type pressure:
            `ArgumentType.Uint`
        """
        self._post_event(11, pressure)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def distance(self, distance: int) -> None:
        """Distance change event

        Sent whenever the distance axis on a tool changes. The value of this
        event is normalized to a value between 0 and 65535.

        Note that distance may be nonzero even when a tool is not in logical
        contact. See the down and up events for more details.

        :param distance:
            The current distance value
        :type distance:
            `ArgumentType.Uint`
        """
        self._post_event(12, distance)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Fixed),
    )
    def tilt(self, tilt_x: float, tilt_y: float) -> None:
        """Tilt change event

        Sent whenever one or both of the tilt axes on a tool change. Each tilt
        value is in degrees, relative to the z-axis of the tablet. The angle is
        positive when the top of a tool tilts along the positive x or y axis.

        :param tilt_x:
            The current value of the X tilt axis
        :type tilt_x:
            `ArgumentType.Fixed`
        :param tilt_y:
            The current value of the Y tilt axis
        :type tilt_y:
            `ArgumentType.Fixed`
        """
        self._post_event(13, tilt_x, tilt_y)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
    )
    def rotation(self, degrees: float) -> None:
        """Z-rotation change event

        Sent whenever the z-rotation axis on the tool changes. The rotation
        value is in degrees clockwise from the tool's logical neutral position.

        :param degrees:
            The current rotation of the Z axis
        :type degrees:
            `ArgumentType.Fixed`
        """
        self._post_event(14, degrees)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Int),
    )
    def slider(self, position: int) -> None:
        """Slider position change event

        Sent whenever the slider position on the tool changes. The value is
        normalized between -65535 and 65535, with 0 as the logical neutral
        position of the slider.

        The slider is available on e.g. the Wacom Airbrush tool.

        :param position:
            The current position of slider
        :type position:
            `ArgumentType.Int`
        """
        self._post_event(15, position)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Fixed),
        Argument(ArgumentType.Int),
    )
    def wheel(self, degrees: float, clicks: int) -> None:
        """Wheel delta event

        Sent whenever the wheel on the tool emits an event. This event contains
        two values for the same axis change. The degrees value is in the same
        orientation as the :func:`WlPointer.vertical_scroll()
        <pywayland.protocol.wayland.WlPointer.vertical_scroll>` axis. The
        clicks value is in discrete logical clicks of the mouse wheel. This
        value may be zero if the movement of the wheel was less than one
        logical click.

        Clients should choose either value and avoid mixing degrees and clicks.
        The compositor may accumulate values smaller than a logical click and
        emulate click events when a certain threshold is met. Thus,
        wl_tablet_tool.wheel events with non-zero clicks values may have
        different degrees values.

        :param degrees:
            The wheel delta in degrees
        :type degrees:
            `ArgumentType.Fixed`
        :param clicks:
            The wheel delta in discrete clicks
        :type clicks:
            `ArgumentType.Int`
        """
        self._post_event(16, degrees, clicks)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def button(self, serial: int, button: int, state: int) -> None:
        """Button event

        Sent whenever a button on the tool is pressed or released.

        If a button is held down when the tool moves in or out of proximity,
        button events are generated by the compositor. See
        wp_tablet_tool.proximity_in and wp_tablet_tool.proximity_out for
        details.

        :param serial:
        :type serial:
            `ArgumentType.Uint`
        :param button:
            The button whose state has changed
        :type button:
            `ArgumentType.Uint`
        :param state:
            Whether the button was pressed or released
        :type state:
            `ArgumentType.Uint`
        """
        self._post_event(17, serial, button, state)

    @ZwpTabletToolV2.event(
        Argument(ArgumentType.Uint),
    )
    def frame(self, time: int) -> None:
        """Frame event

        Marks the end of a series of axis and/or button updates from the
        tablet. The Wayland protocol requires axis updates to be sent
        sequentially, however all events within a frame should be considered
        one hardware event.

        :param time:
            The time of the event with millisecond granularity
        :type time:
            `ArgumentType.Uint`
        """
        self._post_event(18, time)


class ZwpTabletV2Resource(Resource[ZwpTabletV2]):
    interface = ZwpTabletV2

    @ZwpTabletV2.event(
        Argument(ArgumentType.String),
    )
    def name(self, name: str) -> None:
        """Tablet device name

        A descriptive name for the tablet device.

        If the device has no descriptive name, this event is not sent.

        This event is sent in the initial burst of events before the
        wp_tablet.done event.

        :param name:
            the device name
        :type name:
            `ArgumentType.String`
        """
        self._post_event(0, name)

    @ZwpTabletV2.event(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def id(self, vid: int, pid: int) -> None:
        """Tablet device usb vendor/product id

        The USB vendor and product IDs for the tablet device.

        If the device has no USB vendor/product ID, this event is not sent.
        This can happen for virtual devices or non-USB devices, for instance.

        This event is sent in the initial burst of events before the
        wp_tablet.done event.

        :param vid:
            USB vendor id
        :type vid:
            `ArgumentType.Uint`
        :param pid:
            USB product id
        :type pid:
            `ArgumentType.Uint`
        """
        self._post_event(1, vid, pid)

    @ZwpTabletV2.event(
        Argument(ArgumentType.String),
    )
    def path(self, path: str) -> None:
        """Path to the device

        A system-specific device path that indicates which device is behind
        this wp_tablet. This information may be used to gather additional
        information about the device, e.g. through libwacom.

        A device may have more than one device path. If so, multiple
        wp_tablet.path events are sent. A device may be emulated and not have a
        device path, and in that case this event will not be sent.

        The format of the path is unspecified, it may be a device node, a sysfs
        path, or some other identifier. It is up to the client to identify the
        string provided.

        This event is sent in the initial burst of events before the
        wp_tablet.done event.

        :param path:
            path to local device
        :type path:
            `ArgumentType.String`
        """
        self._post_event(2, path)

    @ZwpTabletV2.event()
    def done(self) -> None:
        """Tablet description events sequence complete

        This event is sent immediately to signal the end of the initial burst
        of descriptive events. A client may consider the static description of
        the tablet to be complete and finalize initialization of the tablet.
        """
        self._post_event(3)

    @ZwpTabletV2.event()
    def removed(self) -> None:
        """Tablet removed event

        Sent when the tablet has been removed from the system. When a tablet is
        removed, some tools may be removed.

        When this event is received, the client must wp_tablet.destroy the
        object.
        """
        self._post_event(4)


class ZwpTabletManagerV2Proxy(Proxy[ZwpTabletManagerV2]):
    interface = ZwpTabletManagerV2

    @ZwpTabletManagerV2.request(
        Argument(ArgumentType.NewId, interface=ZwpTabletSeatV2),
        Argument(ArgumentType.Object, interface=WlSeat),
    )
    def get_tablet_seat(self, seat: WlSeatProxy) -> ZwpTabletSeatV2Proxy:
        """Get the tablet seat

        Get the wp_tablet_seat object for the given seat. This object provides
        access to all graphics tablets in this seat.

        :param seat:
            The :class:`~pywayland.protocol.wayland.WlSeat` object to retrieve
            the tablets for
        :type seat:
            :class:`~pywayland.protocol.wayland.WlSeat`
        :returns:
            :class:`~pywayland.protocol.tablet_unstable_v2.ZwpTabletSeatV2`
        """
        tablet_seat = self._marshal_constructor(0, ZwpTabletSeatV2, seat)
        return tablet_seat

    @ZwpTabletManagerV2.request()
    def destroy(self) -> None:
        """Release the memory for the tablet manager object

        Destroy the wp_tablet_manager object. Objects created from this object
        are unaffected and should be destroyed separately.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadGroupV2Proxy(Proxy[ZwpTabletPadGroupV2]):
    interface = ZwpTabletPadGroupV2

    @ZwpTabletPadGroupV2.request()
    def destroy(self) -> None:
        """Destroy the pad object

        Destroy the wp_tablet_pad_group object. Objects created from this
        object are unaffected and should be destroyed separately.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletPadRingV2Proxy(Proxy[ZwpTabletPadRingV2]):
    interface = ZwpTabletPadRingV2

    @ZwpTabletPadRingV2.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, description: str, serial: int) -> None:
        """Set compositor feedback

        Request that the compositor use the provided feedback string associated
        with this ring. This request should be issued immediately after a
        wp_tablet_pad_group.mode_switch event from the corresponding group is
        received, or whenever the ring is mapped to a different action. See
        wp_tablet_pad_group.mode_switch for more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with the ring; compositors may use this information
        to offer visual feedback about the button layout (eg. on-screen
        displays).

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        wp_tablet_pad_group.mode_switch event received for the group of this
        ring. Requests providing other serials than the most recent one will be
        ignored.

        :param description:
            ring description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, description, serial)

    @ZwpTabletPadRingV2.request()
    def destroy(self) -> None:
        """Destroy the ring object

        This destroys the client's resource for this ring object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadStripV2Proxy(Proxy[ZwpTabletPadStripV2]):
    interface = ZwpTabletPadStripV2

    @ZwpTabletPadStripV2.request(
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, description: str, serial: int) -> None:
        """Set compositor feedback

        Requests the compositor to use the provided feedback string associated
        with this strip. This request should be issued immediately after a
        wp_tablet_pad_group.mode_switch event from the corresponding group is
        received, or whenever the strip is mapped to a different action. See
        wp_tablet_pad_group.mode_switch for more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with the strip, and compositors may use this
        information to offer visual feedback about the button layout (eg. on-
        screen displays).

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        wp_tablet_pad_group.mode_switch event received for the group of this
        strip. Requests providing other serials than the most recent one will
        be ignored.

        :param description:
            strip description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, description, serial)

    @ZwpTabletPadStripV2.request()
    def destroy(self) -> None:
        """Destroy the strip object

        This destroys the client's resource for this strip object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletPadV2Proxy(Proxy[ZwpTabletPadV2]):
    interface = ZwpTabletPadV2

    @ZwpTabletPadV2.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String),
        Argument(ArgumentType.Uint),
    )
    def set_feedback(self, button: int, description: str, serial: int) -> None:
        """Set compositor feedback

        Requests the compositor to use the provided feedback string associated
        with this button. This request should be issued immediately after a
        wp_tablet_pad_group.mode_switch event from the corresponding group is
        received, or whenever a button is mapped to a different action. See
        wp_tablet_pad_group.mode_switch for more details.

        Clients are encouraged to provide context-aware descriptions for the
        actions associated with each button, and compositors may use this
        information to offer visual feedback on the button layout (e.g. on-
        screen displays).

        Button indices start at 0. Setting the feedback string on a button that
        is reserved by the compositor (i.e. not belonging to any
        wp_tablet_pad_group) does not generate an error but the compositor is
        free to ignore the request.

        The provided string 'description' is a UTF-8 encoded string to be
        associated with this ring, and is considered user-visible; general
        internationalization rules apply.

        The serial argument will be that of the last
        wp_tablet_pad_group.mode_switch event received for the group of this
        button. Requests providing other serials than the most recent one will
        be ignored.

        :param button:
            button index
        :type button:
            `ArgumentType.Uint`
        :param description:
            button description
        :type description:
            `ArgumentType.String`
        :param serial:
            serial of the mode switch event
        :type serial:
            `ArgumentType.Uint`
        """
        self._marshal(0, button, description, serial)

    @ZwpTabletPadV2.request()
    def destroy(self) -> None:
        """Destroy the pad object

        Destroy the wp_tablet_pad object. Objects created from this object are
        unaffected and should be destroyed separately.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletSeatV2Proxy(Proxy[ZwpTabletSeatV2]):
    interface = ZwpTabletSeatV2

    @ZwpTabletSeatV2.request()
    def destroy(self) -> None:
        """Release the memory for the tablet seat object

        Destroy the wp_tablet_seat object. Objects created from this object are
        unaffected and should be destroyed separately.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletToolV2Proxy(Proxy[ZwpTabletToolV2]):
    interface = ZwpTabletToolV2

    @ZwpTabletToolV2.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Object, interface=WlSurface, nullable=True),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def set_cursor(self, serial: int, surface: WlSurfaceProxy | None, hotspot_x: int, hotspot_y: int) -> None:
        """Set the tablet tool's surface

        Sets the surface of the cursor used for this tool on the given tablet.
        This request only takes effect if the tool is in proximity of one of
        the requesting client's surfaces or the surface parameter is the
        current pointer surface. If there was a previous surface set with this
        request it is replaced. If surface is NULL, the cursor image is hidden.

        The parameters hotspot_x and hotspot_y define the position of the
        pointer surface relative to the pointer location. Its top-left corner
        is always at (x, y) - (hotspot_x, hotspot_y), where (x, y) are the
        coordinates of the pointer location, in surface-local coordinates.

        On surface.attach requests to the pointer surface, hotspot_x and
        hotspot_y are decremented by the x and y parameters passed to the
        request. Attach must be confirmed by :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` as usual.

        The hotspot can also be updated by passing the currently set pointer
        surface to this request with new values for hotspot_x and hotspot_y.

        The current and pending input regions of the
        :class:`~pywayland.protocol.wayland.WlSurface` are cleared, and
        :func:`WlSurface.set_input_region()
        <pywayland.protocol.wayland.WlSurface.set_input_region>` is ignored
        until the :class:`~pywayland.protocol.wayland.WlSurface` is no longer
        used as the cursor. When the use as a cursor ends, the current and
        pending input regions become undefined, and the
        :class:`~pywayland.protocol.wayland.WlSurface` is unmapped.

        This request gives the surface the role of a wp_tablet_tool cursor. A
        surface may only ever be used as the cursor surface for one
        wp_tablet_tool. If the surface already has another role or has
        previously been used as cursor surface for a different tool, a protocol
        error is raised.

        :param serial:
            serial of the proximity_in event
        :type serial:
            `ArgumentType.Uint`
        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface` or `None`
        :param hotspot_x:
            surface-local x coordinate
        :type hotspot_x:
            `ArgumentType.Int`
        :param hotspot_y:
            surface-local y coordinate
        :type hotspot_y:
            `ArgumentType.Int`
        """
        self._marshal(0, serial, surface, hotspot_x, hotspot_y)

    @ZwpTabletToolV2.request()
    def destroy(self) -> None:
        """Destroy the tool object

        This destroys the client's resource for this tool object.
        """
        self._marshal(1)
        self._destroy()


class ZwpTabletV2Proxy(Proxy[ZwpTabletV2]):
    interface = ZwpTabletV2

    @ZwpTabletV2.request()
    def destroy(self) -> None:
        """Destroy the tablet object

        This destroys the client's resource for this tablet object.
        """
        self._marshal(0)
        self._destroy()


class ZwpTabletManagerV2Global(Global[ZwpTabletManagerV2]):
    interface = ZwpTabletManagerV2


class ZwpTabletPadGroupV2Global(Global[ZwpTabletPadGroupV2]):
    interface = ZwpTabletPadGroupV2


class ZwpTabletPadRingV2Global(Global[ZwpTabletPadRingV2]):
    interface = ZwpTabletPadRingV2


class ZwpTabletPadStripV2Global(Global[ZwpTabletPadStripV2]):
    interface = ZwpTabletPadStripV2


class ZwpTabletPadV2Global(Global[ZwpTabletPadV2]):
    interface = ZwpTabletPadV2


class ZwpTabletSeatV2Global(Global[ZwpTabletSeatV2]):
    interface = ZwpTabletSeatV2


class ZwpTabletToolV2Global(Global[ZwpTabletToolV2]):
    interface = ZwpTabletToolV2


class ZwpTabletV2Global(Global[ZwpTabletV2]):
    interface = ZwpTabletV2


ZwpTabletManagerV2._gen_c()
ZwpTabletManagerV2.proxy_class = ZwpTabletManagerV2Proxy
ZwpTabletManagerV2.resource_class = ZwpTabletManagerV2Resource
ZwpTabletManagerV2.global_class = ZwpTabletManagerV2Global


ZwpTabletPadGroupV2._gen_c()
ZwpTabletPadGroupV2.proxy_class = ZwpTabletPadGroupV2Proxy
ZwpTabletPadGroupV2.resource_class = ZwpTabletPadGroupV2Resource
ZwpTabletPadGroupV2.global_class = ZwpTabletPadGroupV2Global


ZwpTabletPadRingV2._gen_c()
ZwpTabletPadRingV2.proxy_class = ZwpTabletPadRingV2Proxy
ZwpTabletPadRingV2.resource_class = ZwpTabletPadRingV2Resource
ZwpTabletPadRingV2.global_class = ZwpTabletPadRingV2Global


ZwpTabletPadStripV2._gen_c()
ZwpTabletPadStripV2.proxy_class = ZwpTabletPadStripV2Proxy
ZwpTabletPadStripV2.resource_class = ZwpTabletPadStripV2Resource
ZwpTabletPadStripV2.global_class = ZwpTabletPadStripV2Global


ZwpTabletPadV2._gen_c()
ZwpTabletPadV2.proxy_class = ZwpTabletPadV2Proxy
ZwpTabletPadV2.resource_class = ZwpTabletPadV2Resource
ZwpTabletPadV2.global_class = ZwpTabletPadV2Global


ZwpTabletSeatV2._gen_c()
ZwpTabletSeatV2.proxy_class = ZwpTabletSeatV2Proxy
ZwpTabletSeatV2.resource_class = ZwpTabletSeatV2Resource
ZwpTabletSeatV2.global_class = ZwpTabletSeatV2Global


ZwpTabletToolV2._gen_c()
ZwpTabletToolV2.proxy_class = ZwpTabletToolV2Proxy
ZwpTabletToolV2.resource_class = ZwpTabletToolV2Resource
ZwpTabletToolV2.global_class = ZwpTabletToolV2Global


ZwpTabletV2._gen_c()
ZwpTabletV2.proxy_class = ZwpTabletV2Proxy
ZwpTabletV2.resource_class = ZwpTabletV2Resource
ZwpTabletV2.global_class = ZwpTabletV2Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2018 The Chromium Authors
# Copyright 2023 Simon Ser
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .tablet_v2 import ZwpTabletToolV2
from .tablet_v2 import ZwpTabletToolV2Proxy
from .wayland import WlPointer
from .wayland import WlPointerProxy


class WpCursorShapeDeviceV1(Interface):
    """Cursor shape for a device

    This interface allows clients to set the cursor shape.
    """

    name = "wp_cursor_shape_device_v1"
    version = 2

    class shape(enum.IntEnum):
        default = 1
        context_menu = 2
        help = 3
        pointer = 4
        progress = 5
        wait = 6
        cell = 7
        crosshair = 8
        text = 9
        vertical_text = 10
        alias = 11
        copy = 12
        move = 13
        no_drop = 14
        not_allowed = 15
        grab = 16
        grabbing = 17
        e_resize = 18
        n_resize = 19
        ne_resize = 20
        nw_resize = 21
        s_resize = 22
        se_resize = 23
        sw_resize = 24
        w_resize = 25
        ew_resize = 26
        ns_resize = 27
        nesw_resize = 28
        nwse_resize = 29
        col_resize = 30
        row_resize = 31
        all_scroll = 32
        zoom_in = 33
        zoom_out = 34
        dnd_ask = 35
        all_resize = 36

    class error(enum.IntEnum):
        invalid_shape = 1


class WpCursorShapeManagerV1(Interface):
    """Cursor shape manager

    This global offers an alternative, optional way to set cursor images. This
    new way uses enumerated cursors instead of a
    :class:`~pywayland.protocol.wayland.WlSurface` like
    :func:`WlPointer.set_cursor()
    <pywayland.protocol.wayland.WlPointer.set_cursor>` does.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_cursor_shape_manager_v1"
    version = 2


class WpCursorShapeDeviceV1Resource(Resource[WpCursorShapeDeviceV1]):
    interface = WpCursorShapeDeviceV1


class WpCursorShapeManagerV1Resource(Resource[WpCursorShapeManagerV1]):
    interface = WpCursorShapeManagerV1


class WpCursorShapeDeviceV1Proxy(Proxy[WpCursorShapeDeviceV1]):
    interface = WpCursorShapeDeviceV1

    @WpCursorShapeDeviceV1.request()
    def destroy(self) -> None:
        """Destroy the cursor shape device

        Destroy the cursor shape device.

        The device cursor shape remains unchanged.
        """
        self._marshal(0)
        self._destroy()

    @WpCursorShapeDeviceV1.request(
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.Uint),
    )
    def set_shape(self, serial: int, shape: int) -> None:
        """Set device cursor to the shape

        Sets the device cursor to the specified shape. The compositor will
        change the cursor image based on the specified shape.

        The cursor actually changes only if the input device focus is one of
        the requesting client's surfaces. If any, the previous cursor image
        (surface or shape) is replaced.

        The "shape" argument must be a valid enum entry, otherwise the
        invalid_shape protocol error is raised.

        This is similar to the :func:`WlPointer.set_cursor()
        <pywayland.protocol.wayland.WlPointer.set_cursor>` and
        :func:`ZwpTabletToolV2.set_cursor()
        <pywayland.protocol.tablet_v2.ZwpTabletToolV2.set_cursor>` requests,
        but this request accepts a shape instead of contents in the form of a
        surface. Clients can mix set_cursor and set_shape requests.

        The serial parameter must match the latest :func:`WlPointer.enter()
        <pywayland.protocol.wayland.WlPointer.enter>` or
        :func:`ZwpTabletToolV2.proximity_in()
        <pywayland.protocol.tablet_v2.ZwpTabletToolV2.proximity_in>` serial
        number sent to the client. Otherwise the request will be ignored.

        :param serial:
            serial number of the enter event
        :type serial:
            `ArgumentType.Uint`
        :param shape:
        :type shape:
            `ArgumentType.Uint`
        """
        self._marshal(1, serial, shape)


class WpCursorShapeManagerV1Proxy(Proxy[WpCursorShapeManagerV1]):
    interface = WpCursorShapeManagerV1

    @WpCursorShapeManagerV1.request()
    def destroy(self) -> None:
        """Destroy the manager

        Destroy the cursor shape manager.
        """
        self._marshal(0)
        self._destroy()

    @WpCursorShapeManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpCursorShapeDeviceV1),
        Argument(ArgumentType.Object, interface=WlPointer),
    )
    def get_pointer(self, pointer: WlPointerProxy) -> WpCursorShapeDeviceV1Proxy:
        """Manage the cursor shape of a pointer device

        Obtain a
        :class:`~pywayland.protocol.cursor_shape_v1.WpCursorShapeDeviceV1` for
        a :class:`~pywayland.protocol.wayland.WlPointer` object.

        When the pointer capability is removed from the
        :class:`~pywayland.protocol.wayland.WlSeat`, the
        :class:`~pywayland.protocol.cursor_shape_v1.WpCursorShapeDeviceV1`
        object becomes inert.

        :param pointer:
        :type pointer:
            :class:`~pywayland.protocol.wayland.WlPointer`
        :returns:
            :class:`~pywayland.protocol.cursor_shape_v1.WpCursorShapeDeviceV1`
        """
        cursor_shape_device = self._marshal_constructor(1, WpCursorShapeDeviceV1, pointer)
        return cursor_shape_device

    @WpCursorShapeManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpCursorShapeDeviceV1),
        Argument(ArgumentType.Object, interface=ZwpTabletToolV2),
    )
    def get_tablet_tool_v2(self, tablet_tool: ZwpTabletToolV2Proxy) -> WpCursorShapeDeviceV1Proxy:
        """Manage the cursor shape of a tablet tool device

        Obtain a
        :class:`~pywayland.protocol.cursor_shape_v1.WpCursorShapeDeviceV1` for
        a :class:`~pywayland.protocol.tablet_v2.ZwpTabletToolV2` object.

        When the :class:`~pywayland.protocol.tablet_v2.ZwpTabletToolV2` is
        removed, the
        :class:`~pywayland.protocol.cursor_shape_v1.WpCursorShapeDeviceV1`
        object becomes inert.

        :param tablet_tool:
        :type tablet_tool:
            :class:`~pywayland.protocol.tablet_v2.ZwpTabletToolV2`
        :returns:
            :class:`~pywayland.protocol.cursor_shape_v1.WpCursorShapeDeviceV1`
        """
        cursor_shape_device = self._marshal_constructor(2, WpCursorShapeDeviceV1, tablet_tool)
        return cursor_shape_device


class WpCursorShapeDeviceV1Global(Global[WpCursorShapeDeviceV1]):
    interface = WpCursorShapeDeviceV1


class WpCursorShapeManagerV1Global(Global[WpCursorShapeManagerV1]):
    interface = WpCursorShapeManagerV1


WpCursorShapeDeviceV1._gen_c()
WpCursorShapeDeviceV1.proxy_class = WpCursorShapeDeviceV1Proxy
WpCursorShapeDeviceV1.resource_class = WpCursorShapeDeviceV1Resource
WpCursorShapeDeviceV1.global_class = WpCursorShapeDeviceV1Global


WpCursorShapeManagerV1._gen_c()
WpCursorShapeManagerV1.proxy_class = WpCursorShapeManagerV1Proxy
WpCursorShapeManagerV1.resource_class = WpCursorShapeManagerV1Resource
WpCursorShapeManagerV1.global_class = WpCursorShapeManagerV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2021 Emmanuel Gil Peyrot
# Copyright © 2022 Xaver Hugl
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlSurface
from .wayland import WlSurfaceProxy


class WpContentTypeManagerV1(Interface):
    """Surface content type manager

    This interface allows a client to describe the kind of content a surface
    will display, to allow the compositor to optimize its behavior for it.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "wp_content_type_manager_v1"
    version = 1

    class error(enum.IntEnum):
        already_constructed = 0


class WpContentTypeV1(Interface):
    """Content type object for a surface

    The content type object allows the compositor to optimize for the kind of
    content shown on the surface. A compositor may for example use it to set
    relevant drm properties like "content type".

    The client may request to switch to another content type at any time. When
    the associated surface gets destroyed, this object becomes inert and the
    client should destroy it.
    """

    name = "wp_content_type_v1"
    version = 1

    class type(enum.IntEnum):
        none = 0
        photo = 1
        video = 2
        game = 3


class WpContentTypeManagerV1Resource(Resource[WpContentTypeManagerV1]):
    interface = WpContentTypeManagerV1


class WpContentTypeV1Resource(Resource[WpContentTypeV1]):
    interface = WpContentTypeV1


class WpContentTypeManagerV1Proxy(Proxy[WpContentTypeManagerV1]):
    interface = WpContentTypeManagerV1

    @WpContentTypeManagerV1.request()
    def destroy(self) -> None:
        """Destroy the content type manager object

        Destroy the content type manager. This doesn't destroy objects created
        with the manager.
        """
        self._marshal(0)
        self._destroy()

    @WpContentTypeManagerV1.request(
        Argument(ArgumentType.NewId, interface=WpContentTypeV1),
        Argument(ArgumentType.Object, interface=WlSurface),
    )
    def get_surface_content_type(self, surface: WlSurfaceProxy) -> WpContentTypeV1Proxy:
        """Create a new content type object

        Create a new content type object associated with the given surface.

        Creating a :class:`~pywayland.protocol.content_type_v1.WpContentTypeV1`
        from a :class:`~pywayland.protocol.wayland.WlSurface` which already has
        one attached is a client error: already_constructed.

        :param surface:
        :type surface:
            :class:`~pywayland.protocol.wayland.WlSurface`
        :returns:
            :class:`~pywayland.protocol.content_type_v1.WpContentTypeV1`
        """
        id = self._marshal_constructor(1, WpContentTypeV1, surface)
        return id


class WpContentTypeV1Proxy(Proxy[WpContentTypeV1]):
    interface = WpContentTypeV1

    @WpContentTypeV1.request()
    def destroy(self) -> None:
        """Destroy the content type object

        Switch back to not specifying the content type of this surface. This is
        equivalent to setting the content type to none, including double
        buffering semantics. See set_content_type for details.
        """
        self._marshal(0)
        self._destroy()

    @WpContentTypeV1.request(
        Argument(ArgumentType.Uint),
    )
    def set_content_type(self, content_type: int) -> None:
        """Specify the content type

        Set the surface content type. This informs the compositor that the
        client believes it is displaying buffers matching this content type.

        This is purely a hint for the compositor, which can be used to adjust
        its behavior or hardware settings to fit the presented content best.

        The content type is double-buffered state, see
        :func:`WlSurface.commit()
        <pywayland.protocol.wayland.WlSurface.commit>` for details.

        :param content_type:
            the content type
        :type content_type:
            `ArgumentType.Uint`
        """
        self._marshal(1, content_type)


class WpContentTypeManagerV1Global(Global[WpContentTypeManagerV1]):
    interface = WpContentTypeManagerV1


class WpContentTypeV1Global(Global[WpContentTypeV1]):
    interface = WpContentTypeV1


WpContentTypeManagerV1._gen_c()
WpContentTypeManagerV1.proxy_class = WpContentTypeManagerV1Proxy
WpContentTypeManagerV1.resource_class = WpContentTypeManagerV1Resource
WpContentTypeManagerV1.global_class = WpContentTypeManagerV1Global


WpContentTypeV1._gen_c()
WpContentTypeV1.proxy_class = WpContentTypeV1Proxy
WpContentTypeV1.resource_class = WpContentTypeV1Resource
WpContentTypeV1.global_class = WpContentTypeV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2023 David Redondo
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .wayland import WlDataSource
from .wayland import WlDataSourceProxy
from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class XdgToplevelDragManagerV1(Interface):
    """Move a window during a drag

    This protocol enhances normal drag and drop with the ability to move a
    window at the same time. This allows having detachable parts of a window
    that when dragged out of it become a new window and can be dragged over an
    existing window to be reattached.

    A typical workflow would be when the user starts dragging on top of a
    detachable part of a window, the client would create a
    :class:`~pywayland.protocol.wayland.WlDataSource` and a
    :class:`~pywayland.protocol.xdg_toplevel_drag_v1.XdgToplevelDragV1` object
    and start the drag as normal via :func:`WlDataDevice.start_drag()
    <pywayland.protocol.wayland.WlDataDevice.start_drag>`. Once the client
    determines that the detachable window contents should be detached from the
    originating window, it creates a new
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` with these contents and
    issues a :func:`XdgToplevelDragV1.attach()
    <pywayland.protocol.xdg_toplevel_drag_v1.XdgToplevelDragV1.attach>` request
    before mapping it. From now on the new window is moved by the compositor
    during the drag as if the client called :func:`XdgToplevel.move()
    <pywayland.protocol.xdg_shell.XdgToplevel.move>`.

    Dragging an existing window is similar. The client creates a
    :class:`~pywayland.protocol.xdg_toplevel_drag_v1.XdgToplevelDragV1` object
    and attaches the existing toplevel before starting the drag.

    Clients use the existing drag and drop mechanism to detect when a window
    can be docked or undocked. If the client wants to snap a window into a
    parent window it should delete or unmap the dragged top-level. If the
    contents should be detached again it attaches a new toplevel as described
    above. If a drag operation is cancelled without being dropped, clients
    should revert to the previous state, deleting any newly created windows as
    appropriate. When a drag operation ends as indicated by
    :func:`WlDataSource.dnd_drop_performed()
    <pywayland.protocol.wayland.WlDataSource.dnd_drop_performed>` the dragged
    toplevel window's final position is determined as if a xdg_toplevel_move
    operation ended.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "xdg_toplevel_drag_manager_v1"
    version = 1

    class error(enum.IntEnum):
        invalid_source = 0


class XdgToplevelDragV1(Interface):
    """Object representing a toplevel move during a drag
    """

    name = "xdg_toplevel_drag_v1"
    version = 1

    class error(enum.IntEnum):
        toplevel_attached = 0
        ongoing_drag = 1


class XdgToplevelDragManagerV1Resource(Resource[XdgToplevelDragManagerV1]):
    interface = XdgToplevelDragManagerV1


class XdgToplevelDragV1Resource(Resource[XdgToplevelDragV1]):
    interface = XdgToplevelDragV1


class XdgToplevelDragManagerV1Proxy(Proxy[XdgToplevelDragManagerV1]):
    interface = XdgToplevelDragManagerV1

    @XdgToplevelDragManagerV1.request()
    def destroy(self) -> None:
        """Destroy the :class:`XdgToplevelDragManagerV1` object

        Destroy this :class:`XdgToplevelDragManagerV1` object. Other objects,
        including
        :class:`~pywayland.protocol.xdg_toplevel_drag_v1.XdgToplevelDragV1`
        objects created by this factory, are not affected by this request.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevelDragManagerV1.request(
        Argument(ArgumentType.NewId, interface=XdgToplevelDragV1),
        Argument(ArgumentType.Object, interface=WlDataSource),
    )
    def get_xdg_toplevel_drag(self, data_source: WlDataSourceProxy) -> XdgToplevelDragV1Proxy:
        """Get an xdg_toplevel_drag for a :class:`~pywayland.protocol.wayland.WlDataSource`

        Create an xdg_toplevel_drag for a drag and drop operation that is going
        to be started with data_source.

        This request can only be made on sources used in drag-and-drop, so it
        must be performed before :func:`WlDataDevice.start_drag()
        <pywayland.protocol.wayland.WlDataDevice.start_drag>`. Attempting to
        use the source other than for drag-and-drop such as in
        :func:`WlDataDevice.set_selection()
        <pywayland.protocol.wayland.WlDataDevice.set_selection>` will raise an
        invalid_source error.

        Destroying data_source while a toplevel is attached to the
        xdg_toplevel_drag is undefined.

        :param data_source:
        :type data_source:
            :class:`~pywayland.protocol.wayland.WlDataSource`
        :returns:
            :class:`~pywayland.protocol.xdg_toplevel_drag_v1.XdgToplevelDragV1`
        """
        id = self._marshal_constructor(1, XdgToplevelDragV1, data_source)
        return id


class XdgToplevelDragV1Proxy(Proxy[XdgToplevelDragV1]):
    interface = XdgToplevelDragV1

    @XdgToplevelDragV1.request()
    def destroy(self) -> None:
        """Destroy an :class:`XdgToplevelDragV1` object

        Destroy this :class:`XdgToplevelDragV1` object. This request must only
        be called after the underlying
        :class:`~pywayland.protocol.wayland.WlDataSource` drag has ended, as
        indicated by the dnd_drop_performed or cancelled events. In any other
        case an ongoing_drag error is raised.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevelDragV1.request(
        Argument(ArgumentType.Object, interface=XdgToplevel),
        Argument(ArgumentType.Int),
        Argument(ArgumentType.Int),
    )
    def attach(self, toplevel: XdgToplevelProxy, x_offset: int, y_offset: int) -> None:
        """Move a toplevel with the drag operation

        Request that the window will be moved with the cursor during the drag
        operation. The offset is a hint to the compositor how the toplevel
        should be positioned relative to the cursor hotspot in surface local
        coordinates and relative to the geometry of the toplevel being
        attached. See :func:`XdgSurface.set_window_geometry()
        <pywayland.protocol.xdg_shell.XdgSurface.set_window_geometry>`. For
        example it might only be used when an unmapped window is attached. The
        attached window does not participate in the selection of the drag
        target.

        If the toplevel is unmapped while it is attached, it is automatically
        detached from the drag. In this case this request has to be called
        again if the window should be attached after it is remapped.

        This request can be called multiple times but issuing it while a
        toplevel with an active role is attached raises a toplevel_attached
        error.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :param x_offset:
            dragged surface x offset
        :type x_offset:
            `ArgumentType.Int`
        :param y_offset:
            dragged surface y offset
        :type y_offset:
            `ArgumentType.Int`
        """
        self._marshal(1, toplevel, x_offset, y_offset)


class XdgToplevelDragManagerV1Global(Global[XdgToplevelDragManagerV1]):
    interface = XdgToplevelDragManagerV1


class XdgToplevelDragV1Global(Global[XdgToplevelDragV1]):
    interface = XdgToplevelDragV1


XdgToplevelDragManagerV1._gen_c()
XdgToplevelDragManagerV1.proxy_class = XdgToplevelDragManagerV1Proxy
XdgToplevelDragManagerV1.resource_class = XdgToplevelDragManagerV1Resource
XdgToplevelDragManagerV1.global_class = XdgToplevelDragManagerV1Global


XdgToplevelDragV1._gen_c()
XdgToplevelDragV1.proxy_class = XdgToplevelDragV1Proxy
XdgToplevelDragV1.resource_class = XdgToplevelDragV1Resource
XdgToplevelDragV1.global_class = XdgToplevelDragV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright © 2023 Carlos Garnacho
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class XdgDialogV1(Interface):
    """Dialog object

    A :class:`XdgDialogV1` object is an ancillary object tied to a
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel`. Its purpose is hinting
    the compositor that the toplevel is a "dialog" (e.g. a temporary window)
    relative to another toplevel (see :func:`XdgToplevel.set_parent()
    <pywayland.protocol.xdg_shell.XdgToplevel.set_parent>`). If the
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` is destroyed, the
    :class:`XdgDialogV1` becomes inert.

    Through this object, the client may provide additional hints about the
    purpose of the secondary toplevel. This interface has no effect on
    toplevels that are not attached to a parent toplevel.
    """

    name = "xdg_dialog_v1"
    version = 1


class XdgWmDialogV1(Interface):
    """Create dialogs related to other toplevels

    The :class:`XdgWmDialogV1` interface is exposed as a global object allowing
    to register surfaces with a
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` role as "dialogs"
    relative to another toplevel.

    The compositor may let this relation influence how the surface is placed,
    displayed or interacted with.

    Warning! The protocol described in this file is currently in the testing
    phase. Backward compatible changes may be added together with the
    corresponding interface version bump. Backward incompatible changes can
    only be done by creating a new major version of the extension.
    """

    name = "xdg_wm_dialog_v1"
    version = 1

    class error(enum.IntEnum):
        already_used = 0


class XdgDialogV1Resource(Resource[XdgDialogV1]):
    interface = XdgDialogV1


class XdgWmDialogV1Resource(Resource[XdgWmDialogV1]):
    interface = XdgWmDialogV1


class XdgDialogV1Proxy(Proxy[XdgDialogV1]):
    interface = XdgDialogV1

    @XdgDialogV1.request()
    def destroy(self) -> None:
        """Destroy the dialog object

        Destroys the :class:`XdgDialogV1` object. If this object is destroyed
        before the related :class:`~pywayland.protocol.xdg_shell.XdgToplevel`,
        the compositor should unapply its effects.
        """
        self._marshal(0)
        self._destroy()

    @XdgDialogV1.request()
    def set_modal(self) -> None:
        """Mark dialog as modal

        Hints that the dialog has "modal" behavior. Modal dialogs typically
        require to be fully addressed by the user (i.e. closed) before resuming
        interaction with the parent toplevel, and may require a distinct
        presentation.

        Clients must implement the logic to filter events in the parent
        toplevel on their own.

        Compositors may choose any policy in event delivery to the parent
        toplevel, from delivering all events unfiltered to using them for
        internal consumption.
        """
        self._marshal(1)

    @XdgDialogV1.request()
    def unset_modal(self) -> None:
        """Mark dialog as not modal

        Drops the hint that this dialog has "modal" behavior. See
        :func:`XdgDialogV1.set_modal()` for more details.
        """
        self._marshal(2)


class XdgWmDialogV1Proxy(Proxy[XdgWmDialogV1]):
    interface = XdgWmDialogV1

    @XdgWmDialogV1.request()
    def destroy(self) -> None:
        """Destroy the dialog manager object

        Destroys the :class:`XdgWmDialogV1` object. This does not affect the
        :class:`~pywayland.protocol.xdg_dialog_v1.XdgDialogV1` objects
        generated through it.
        """
        self._marshal(0)
        self._destroy()

    @XdgWmDialogV1.request(
        Argument(ArgumentType.NewId, interface=XdgDialogV1),
        Argument(ArgumentType.Object, interface=XdgToplevel),
    )
    def get_xdg_dialog(self, toplevel: XdgToplevelProxy) -> XdgDialogV1Proxy:
        """Create a dialog object

        Creates a :class:`~pywayland.protocol.xdg_dialog_v1.XdgDialogV1` object
        for the given toplevel. See the interface description for more details.

        Compositors must raise an already_used error if clients attempt to
        create multiple :class:`~pywayland.protocol.xdg_dialog_v1.XdgDialogV1`
        objects for the same
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel`.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :returns:
            :class:`~pywayland.protocol.xdg_dialog_v1.XdgDialogV1`
        """
        id = self._marshal_constructor(1, XdgDialogV1, toplevel)
        return id


class XdgDialogV1Global(Global[XdgDialogV1]):
    interface = XdgDialogV1


class XdgWmDialogV1Global(Global[XdgWmDialogV1]):
    interface = XdgWmDialogV1


XdgDialogV1._gen_c()
XdgDialogV1.proxy_class = XdgDialogV1Proxy
XdgDialogV1.resource_class = XdgDialogV1Resource
XdgDialogV1.global_class = XdgDialogV1Global


XdgWmDialogV1._gen_c()
XdgWmDialogV1.proxy_class = XdgWmDialogV1Proxy
XdgWmDialogV1.resource_class = XdgWmDialogV1Resource
XdgWmDialogV1.global_class = XdgWmDialogV1Global



# This file has been autogenerated by the pywayland scanner

# Copyright 2018 Mike Blumenkrantz
# Copyright 2018 Samsung Electronics Co., Ltd
# Copyright 2018 Red Hat Inc.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the "Software"),
# to deal in the Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish, distribute, sublicense,
# and/or sell copies of the Software, and to permit persons to whom the
# Software is furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice (including the next
# paragraph) shall be included in all copies or substantial portions of the
# Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
# THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
# DEALINGS IN THE SOFTWARE.

from __future__ import annotations

import enum

from pywayland.protocol_core import Argument, ArgumentType, Global, Interface, Proxy, Resource

from .xdg_shell import XdgToplevel
from .xdg_shell import XdgToplevelProxy


class XdgSessionManagerV1(Interface):
    """Manage sessions for applications

    The :class:`XdgSessionManagerV1` interface defines base requests for
    creating and managing a session for an application. Sessions persist across
    application and compositor restarts unless explicitly destroyed. A session
    is created for the purpose of maintaining an application's
    :class:`~pywayland.protocol.xdg_shell.XdgToplevel` surfaces across
    compositor or application restarts. The compositor should remember as many
    states as possible for surfaces in a given session, but there is no
    requirement for which states must be remembered.

    Policies such as cache eviction are declared an implementation detail of
    the compositor. Clients should account for no longer existing sessions.
    """

    name = "xdg_session_manager_v1"
    version = 1

    class error(enum.IntEnum):
        in_use = 1
        invalid_session_id = 2
        invalid_reason = 3

    class reason(enum.IntEnum):
        launch = 1
        recover = 2
        session_restore = 3


class XdgSessionV1(Interface):
    """A session for an application

    A :class:`XdgSessionV1` object represents a session for an application.
    While the object exists, all surfaces which have been added to the session
    will have states stored by the compositor which can be reapplied at a later
    time. Two sessions cannot exist for the same identifier string.

    States for surfaces added to a session are automatically updated by the
    compositor when they are changed.
    """

    name = "xdg_session_v1"
    version = 1

    class error(enum.IntEnum):
        name_in_use = 1
        already_mapped = 2
        invalid_name = 3
        already_added = 4


class XdgToplevelSessionV1(Interface):
    """A session for an application

    A :class:`XdgToplevelSessionV1` resource acts as a handle for the given
    toplevel in the session. It allows for receiving events after a toplevel
    state was restored, and has the requests to manage them.
    """

    name = "xdg_toplevel_session_v1"
    version = 1


class XdgSessionManagerV1Resource(Resource[XdgSessionManagerV1]):
    interface = XdgSessionManagerV1


class XdgSessionV1Resource(Resource[XdgSessionV1]):
    interface = XdgSessionV1

    @XdgSessionV1.event(
        Argument(ArgumentType.String),
    )
    def created(self, session_id: str) -> None:
        """Newly-created session id

        Emitted at most once some time after getting a new session object. It
        means that no previous state was restored, and a new session was
        created. The passed id can be persistently stored and used to restore
        previous sessions.

        :param session_id:
        :type session_id:
            `ArgumentType.String`
        """
        self._post_event(0, session_id)

    @XdgSessionV1.event()
    def restored(self) -> None:
        """The session has been restored

        Emitted at most once some time after getting a new session object. It
        means that previous state was at least partially restored. The same id
        can again be used to restore previous sessions.
        """
        self._post_event(1)

    @XdgSessionV1.event()
    def replaced(self) -> None:
        """The session has been replaced

        Emitted at most once, if the session was taken over by some other
        client. When this happens, the session and all its toplevel session
        objects become inert, and should be destroyed.
        """
        self._post_event(2)


class XdgToplevelSessionV1Resource(Resource[XdgToplevelSessionV1]):
    interface = XdgToplevelSessionV1

    @XdgToplevelSessionV1.event()
    def restored(self) -> None:
        """A toplevel's session has been restored

        The "restored" event is emitted prior to the first
        :func:`XdgToplevel.configure()
        <pywayland.protocol.xdg_shell.XdgToplevel.configure>` for the toplevel.
        It will only be emitted after :func:`XdgSessionV1.restore_toplevel()
        <pywayland.protocol.xdg_session_management_v1.XdgSessionV1.restore_toplevel>`,
        and the initial empty surface state has been applied, and it indicates
        that the surface's session is being restored with this configure event.
        """
        self._post_event(0)


class XdgSessionManagerV1Proxy(Proxy[XdgSessionManagerV1]):
    interface = XdgSessionManagerV1

    @XdgSessionManagerV1.request()
    def destroy(self) -> None:
        """Destroy this object

        Destroy the manager object. The existing session objects will be
        unaffected.
        """
        self._marshal(0)
        self._destroy()

    @XdgSessionManagerV1.request(
        Argument(ArgumentType.NewId, interface=XdgSessionV1),
        Argument(ArgumentType.Uint),
        Argument(ArgumentType.String, nullable=True),
    )
    def get_session(self, reason: int, session_id: str | None) -> XdgSessionV1Proxy:
        """Create or restore a session

        Create a session object corresponding to either an existing session
        identified by the given session identifier string or a new session.
        While the session object exists, the session is considered to be "in
        use".

        If an identifier string represents a session that is currently actively
        in use by the the same client, an 'in_use' error is raised. If some
        other client is currently using the same session, the new session will
        replace managing the associated state.

        If the reason is not a valid enum entry, the 'invalid_reason' protocol
        error is raised.

        NULL is passed to initiate a new session. If a session_id is passed
        which does not represent a valid session, the compositor treats it as
        if NULL had been passed.

        The session id string must be UTF-8 encoded. It is also limited by the
        maximum length of wayland messages (around 4KB). The
        'invalid_session_id' protocol error will be raised if an invalid string
        is provided.

        A client is allowed to have any number of in use sessions at the same
        time.

        :param reason:
            reason for session
        :type reason:
            `ArgumentType.Uint`
        :param session_id:
            the session to restore
        :type session_id:
            `ArgumentType.String` or `None`
        :returns:
            :class:`~pywayland.protocol.xdg_session_management_v1.XdgSessionV1`
        """
        id = self._marshal_constructor(1, XdgSessionV1, reason, session_id)
        return id


class XdgSessionV1Proxy(Proxy[XdgSessionV1]):
    interface = XdgSessionV1

    @XdgSessionV1.request()
    def destroy(self) -> None:
        """Destroy the session

        Destroy a session object, preserving the current state but not
        continuing to make further updates if state changes occur. This makes
        the associated
        :class:`~pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1`
        objects inert.
        """
        self._marshal(0)
        self._destroy()

    @XdgSessionV1.request()
    def remove(self) -> None:
        """Remove the session

        Remove the session, making it no longer available for restoration. A
        compositor should in response to this request remove the data related
        to this session from its storage.
        """
        self._marshal(1)
        self._destroy()

    @XdgSessionV1.request(
        Argument(ArgumentType.NewId, interface=XdgToplevelSessionV1),
        Argument(ArgumentType.Object, interface=XdgToplevel),
        Argument(ArgumentType.String),
    )
    def add_toplevel(self, toplevel: XdgToplevelProxy, name: str) -> XdgToplevelSessionV1Proxy:
        """Add a new surface to the session

        Attempt to add a given surface to the session. The passed name is used
        to identify what window is being restored, and may be used to store
        window specific state within the session.

        The name given to the toplevel must not correspond to any previously
        existing toplevel names in the session. If the name matches an already
        known toplevel name in the session, a 'name_in_use' protocol error will
        be raised.

        The toplevel object must not be added more than once to any session
        created by the client, otherwise the 'already_added' protocol error
        will be raised.

        This request will return a
        :class:`~pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1`
        for later manipulation. As this resource is created from an empty
        initial state, compositors must not emit a
        :func:`XdgToplevelSessionV1.restored()
        <pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1.restored>`
        event for resources created through this request.

        The name string must be UTF-8 encoded. It is also limited by the
        maximum length of wayland messages (around 4KB). The 'invalid_name'
        protocol error will be raised if an invalid string is provided.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :param name:
            name identifying the toplevel
        :type name:
            `ArgumentType.String`
        :returns:
            :class:`~pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1`
        """
        id = self._marshal_constructor(2, XdgToplevelSessionV1, toplevel, name)
        return id

    @XdgSessionV1.request(
        Argument(ArgumentType.NewId, interface=XdgToplevelSessionV1),
        Argument(ArgumentType.Object, interface=XdgToplevel),
        Argument(ArgumentType.String),
    )
    def restore_toplevel(self, toplevel: XdgToplevelProxy, name: str) -> XdgToplevelSessionV1Proxy:
        """Restore a surface state

        Inform the compositor that the toplevel associated with the passed name
        should have its window management state restored.

        If the toplevel name was previously granted to another
        :class:`~pywayland.protocol.xdg_shell.XdgToplevel`, the 'name_in_use'
        protocol error will be raised.

        The toplevel object must not be added more than once to any session
        created by the client, otherwise the 'already_added' protocol error
        will be raised.

        This request must be called prior to the first commit on the associated
        :class:`~pywayland.protocol.wayland.WlSurface` after creating the
        toplevel, otherwise an 'already_mapped' error is raised.

        As part of the initial configure sequence, if the toplevel was
        successfully restored, a :func:`XdgToplevelSessionV1.restored()
        <pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1.restored>`
        event is emitted. If the toplevel name was not known in the session,
        this request will be equivalent to the
        :func:`XdgToplevelSessionV1.add_toplevel()
        <pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1.add_toplevel>`
        request, and no such event will be emitted. See the
        :func:`XdgToplevelSessionV1.restored()
        <pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1.restored>`
        event for further details.

        The name string must be UTF-8 encoded. It is also limited by the
        maximum length of wayland messages (around 4KB). The 'invalid_name'
        protocol error will be raised if an invalid string is provided.

        :param toplevel:
        :type toplevel:
            :class:`~pywayland.protocol.xdg_shell.XdgToplevel`
        :param name:
            name identifying the toplevel
        :type name:
            `ArgumentType.String`
        :returns:
            :class:`~pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1`
        """
        id = self._marshal_constructor(3, XdgToplevelSessionV1, toplevel, name)
        return id

    @XdgSessionV1.request(
        Argument(ArgumentType.String),
    )
    def remove_toplevel(self, name: str) -> None:
        """Remove a surface from the session

        Remove a specified surface from the session and render any related
        :class:`~pywayland.protocol.xdg_session_management_v1.XdgToplevelSessionV1`
        object inert. The compositor should remove any data related to the
        toplevel in the corresponding session from its internal storage.

        The window is specified by its name in the session. The name string
        must be encoded in UTF-8, and it is limited in size by the maximum
        length of wayland messages (around 4KB).

        :param name:
            name identifying the toplevel
        :type name:
            `ArgumentType.String`
        """
        self._marshal(4, name)


class XdgToplevelSessionV1Proxy(Proxy[XdgToplevelSessionV1]):
    interface = XdgToplevelSessionV1

    @XdgToplevelSessionV1.request()
    def destroy(self) -> None:
        """Destroy the object

        Destroy the object. This has no effect over window management of the
        associated toplevel.
        """
        self._marshal(0)
        self._destroy()

    @XdgToplevelSessionV1.request(
        Argument(ArgumentType.String),
    )
    def rename(self, name: str) -> None:
        """Change the name of toplevel session

        Renames the toplevel session. The new name can be used in subsequent
        requests to identify this session object. The state associated with
        this toplevel session will be preserved.

        If the
        :class:`~pywayland.protocol.xdg_session_management_v1.XdgSessionV1`
        already contains a toplevel with the specified name, the 'name_in_use'
        protocol error will be raised.

        :param name:
            new name to identify the toplevel
        :type name:
            `ArgumentType.String`
        """
        self._marshal(1, name)


class XdgSessionManagerV1Global(Global[XdgSessionManagerV1]):
    interface = XdgSessionManagerV1


class XdgSessionV1Global(Global[XdgSessionV1]):
    interface = XdgSessionV1


class XdgToplevelSessionV1Global(Global[XdgToplevelSessionV1]):
    interface = XdgToplevelSessionV1


XdgSessionManagerV1._gen_c()
XdgSessionManagerV1.proxy_class = XdgSessionManagerV1Proxy
XdgSessionManagerV1.resource_class = XdgSessionManagerV1Resource
XdgSessionManagerV1.global_class = XdgSessionManagerV1Global


XdgSessionV1._gen_c()
XdgSessionV1.proxy_class = XdgSessionV1Proxy
XdgSessionV1.resource_class = XdgSessionV1Resource
XdgSessionV1.global_class = XdgSessionV1Global


XdgToplevelSessionV1._gen_c()
XdgToplevelSessionV1.proxy_class = XdgToplevelSessionV1Proxy
XdgToplevelSessionV1.resource_class = XdgToplevelSessionV1Resource
XdgToplevelSessionV1.global_class = XdgToplevelSessionV1Global



# Copyright 2015 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

from logging import getLogger
from typing import TYPE_CHECKING

from pywayland import ffi, lib
from pywayland.utils import wl_container_of

if TYPE_CHECKING:
    from collections.abc import Callable
    from typing import Any, ParamSpec, TypeVar

    P = ParamSpec("P")
    R = TypeVar("R")

logger = getLogger(__package__)


# void (*wl_notify_func_t)(struct wl_listener *listener, void *data);
@ffi.def_extern()
def notify_func(listener_ptr: ffi.WlListener, data: ffi.CData) -> None:
    container: ffi.WlListenerContainer = wl_container_of(
        listener_ptr, "struct wl_listener_container *", "destroy_listener"
    )
    listener = ffi.from_handle(container.handle)

    if listener._signal is not None and listener._signal._data_wrapper is not None:
        data = listener._signal._data_wrapper(data)

    callback = listener._notify

    try:
        callback(listener, data)
    except Exception:
        logger.exception("Exception in callback function")


class Listener:
    """A single listener for Wayland signals

    Provides the means to listen for `wl_listener` signal notifications.  Many
    Wayland objects use `wl_listener` for notification of significant events
    like object destruction.

    Clients should create :class:`Listener` objects manually and can register
    them as listeners to objects destroy events using the object's
    ``.add_destroy_listener()`` method.  A listener can only listen to one
    signal at a time.

    :param function: callback function for the Listener
    :type function: callable
    """

    def __init__(self, function: Callable[P, R]) -> None:
        self._ptr: ffi.WlListener | None
        self._handle: ffi.CData = ffi.new_handle(self)

        # we need a way to get this Python object from the `struct
        # wl_listener*`, so we put the pointer in a container struct that
        # contains both the wl_listener and a pointer to our ffi handle
        self.container: ffi.WlListenerContainer = ffi.new(
            "struct wl_listener_container *"
        )
        self.container.handle = self._handle

        self._ptr = ffi.addressof(self.container.destroy_listener)
        self._ptr.notify = lib.notify_func  # type: ignore [assignment]
        self._notify = function
        self._signal: Signal | None = None

    def remove(self) -> None:
        """Remove the listener"""
        if self._ptr and self._ptr.link != ffi.NULL:
            lib.wl_list_remove(ffi.addressof(self._ptr.link))
            self._ptr = None


class Signal:
    """A source of a type of observable event

    Signals are recognized points where significant events can be observed.
    Compositors as well as the server can provide signals. Observers are
    wl_listener's that are added through #wl_signal_add. Signals are emitted
    using #wl_signal_emit, which will invoke all listeners until that listener
    is removed by wl_list_remove() (or whenever the signal is destroyed).
    """

    def __init__(
        self,
        *,
        ptr: ffi.WlSignal | None = None,
        data_wrapper: Callable[P, R] | None = None,
    ) -> None:
        if ptr is None:
            self._ptr: ffi.WlSignal = ffi.new("struct wl_signal *")
            lib.wl_signal_init(self._ptr)
        else:
            self._ptr = ptr

        self._data_wrapper = data_wrapper
        self._link: list[Listener] = []

    def add(self, listener: Listener) -> None:
        """Add the specified listener to this signal

        :param listener: The listener to add
        :type listener: :class:`Listener`
        """
        # we are creating a reference loop, as the listener resolves the data
        # wrapper function, and the signal keeps the listener alive
        listener._signal = self
        self._link.append(listener)

        if listener._ptr is not None:
            lib.wl_signal_add(self._ptr, listener._ptr)

    def emit(self, data: Any = None) -> None:
        """Emits this signal, notifying all registered listeners

        :param data: The data that will be emitted with the signal
        """
        data_ptr: ffi.CData
        if data is not None:
            data_ptr = ffi.new_handle(data)
        else:
            data_ptr = ffi.NULL
        lib.wl_signal_emit(self._ptr, data_ptr)

    def remove_listeners(self) -> None:
        """Removes all listeners from this signal.

        This can be useful when an object is about to be destroyed
        and all listeners need to be removed from that object.
        """
        for listener in self._link[:]:
            listener.remove()
